package com.ruoyi.ss.device.service.impl;

import com.ruoyi.common.core.domain.BaseValidator;
import com.ruoyi.common.core.domain.ValidateResult;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.device.domain.DeviceBO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.enums.DeviceServiceMode;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.model.domain.enums.ModelTag;
import com.ruoyi.ss.store.service.StoreValidator;
import com.ruoyi.ss.suit.service.SuitValidator;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import org.eclipse.jetty.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author wjh
 * 2024/4/30
 */
@Service
public class DeviceValidatorImpl extends BaseValidator implements DeviceValidator {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private StoreValidator storeValidator;

    @Autowired
    private SuitValidator suitValidator;

    @Autowired
    private TransactionBillService transactionBillService;

    /**
     * 设备编号是否存在
     *
     * @param deviceNos
     */
    @Override
    public boolean isExistNo(List<String> deviceNos) {
        if (CollectionUtils.isEmptyElement(deviceNos)) {
            return true;
        }
        DeviceQuery query = new DeviceQuery();
        query.setDeviceNos(deviceNos);
        return deviceService.selectCount(query) == new HashSet<>(deviceNos).size();
    }

    /**
     * 判断设备是否属于指定用户
     *
     * @param deviceId 设备id
     * @param userId   用户id
     */
    @Override
    public boolean isBelong(Long deviceId, Long userId) {
        if (deviceId == null || userId == null) {
            return false;
        }
        DeviceVO device = deviceService.selectById(deviceId);
        return device != null && Objects.equals(device.getUserId(), userId);
    }

    @Override
    public boolean isBelong(Collection<Long> deviceIds, Long userId) {
        if (CollectionUtils.isEmptyElement(deviceIds) || userId == null) {
            return false;
        }
        DeviceQuery query = new DeviceQuery();
        query.setDeviceIds(new ArrayList<>(deviceIds));
        query.setUserId(userId);
        List<DeviceVO> deviceList = deviceService.selectSmDeviceList(query);
        return deviceList.size() == deviceIds.size();
    }

    /**
     * 创建前校验
     *
     * @param data
     * @return
     */
    @Override
    public ValidateResult preCreate(DeviceBO data) {
        if (data == null) {
            return error("数据不允许为空");
        }
        return success();
    }

    /**
     * 更新前校验
     *
     * @param data
     */
    @Override
    public ValidateResult preUpdate(DeviceBO data) {
        if (data == null) {
            return error("数据不允许为空");
        }
        return success();
    }

    /**
     * APP更新前校验
     *
     * @param data
     */
    @Override
    public ValidateResult preUpdateByApp(DeviceBO data) {
        if (data == null) {
            return error("数据不允许为空");
        }

        if (data.getStoreId() != null && !storeValidator.isExist(Collections.singletonList(data.getStoreId()))) {
            return error("店铺不存在");
        }

        Long userId = SecurityUtils.getUserId();

        // 判断是否有权限更改
        if (!isBelong(data.getDeviceId(), userId)) {
            return error("这不是您的设备，您无权更改");
        }
        // 判断套餐
        if (CollectionUtils.isNotEmptyElement(data.getSuitIds()) && !suitValidator.isBelong(data.getSuitIds(), userId)) {
            return error("有套餐不属于您，无权修改");
        }

        return success();
    }

    @Override
    public boolean isRepeatSn(Long deviceId, String sn) {
        if (StringUtil.isBlank(sn)) {
            return false;
        }
        DeviceVO device = deviceService.selectByDeviceNo(sn);
        return device != null && !Objects.equals(deviceId, device.getDeviceId());
    }

    @Override
    public boolean isRepeatMac(Long deviceId, String mac) {
        if (StringUtils.isBlank(mac)) {
            return false;
        }
        DeviceQuery query = new DeviceQuery();
        query.setAnyMac(mac);
        List<DeviceVO> repeatList = deviceService.selectSmDeviceList(query);
        if (CollectionUtils.isNotEmptyElement(repeatList)) {
            for (DeviceVO device : repeatList) {
                boolean repeat = device != null && !Objects.equals(deviceId, device.getDeviceId());
                if (repeat) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public void afterCheck(DeviceVO vo) {
        if (vo == null) {
            return;
        }
        if (StringUtils.hasText(vo.getMac()) || StringUtils.hasText(vo.getMac2())) {
            ServiceUtil.assertion(Objects.equals(vo.getMac(), vo.getMac2()), "MAC-1和MAC-2不允许相同");
        }
//        if (vo.getUserId() != null || vo.getAgentId() != null) {
//            ServiceUtil.assertion(Objects.equals(vo.getUserId(), vo.getAgentId()), "代理商和商户不允许是同一个用户");
//        }
        ServiceUtil.assertion(this.isRepeatMac(vo.getDeviceId(), vo.getMac()), "MAC-1重复:" + vo.getMac());
        ServiceUtil.assertion(this.isRepeatMac(vo.getDeviceId(), vo.getMac2()), "MAC-2重复:" + vo.getMac2());
        ServiceUtil.assertion(this.isRepeatSn(vo.getDeviceId(), vo.getDeviceNo()), "SN重复");

        // 校验代理商
        this.checkAgent(vo);

    }

    // 校验代理商
    @Override
    public void checkAgent(DeviceVO vo) {
        if (vo == null) {
            return;
        }
        if (DeviceServiceMode.AGENT.getMode().equals(vo.getServiceMode())) {
            ServiceUtil.assertion(vo.getAgentId() == null, "代理商不允许为空");
            ServiceUtil.assertion(vo.getAgentServiceRate() == null || vo.getAgentUserServiceRate() == null, "设备代理商服务费或平台代理商服务费不允许为空");
            ServiceUtil.assertion(vo.getAgentServiceRate().compareTo(vo.getAgentUserServiceRate()) < 0,
                    "设备服务费不允许小于平台收取代理商的服务费：" + vo.getAgentUserServiceRate() + "%");
        }
    }

    @Override
    public boolean isAgent(DeviceVO device, Long userId) {
        if (device == null || userId == null) {
            return false;
        }
        return Objects.equals(device.getAgentId(), userId);
    }

    @Override
    public boolean hasUsingBill(Long deviceId) {
        if (deviceId == null) {
            return false;
        }
        TransactionBillQuery query = new TransactionBillQuery();
        query.setDeviceId(deviceId);
        query.setType(TransactionBillType.RECHARGE.getType());
        query.setIsUsing(true);
        int count = transactionBillService.selectSimpleCount(query);
        return count > 0;
    }
}
